/*

 Fade-in LED driver
  MPU: ATtiny402
  LED: SK6805-2427

 MaiaR create
  maiar_create@zeus.eonet.ne.jp

 Tresure Chest Ver3.3
  http://ikkei.akiba.coocan.jp/ikkei_Electronics/treasure.html
  
*/


#include <avr/sleep.h>
#include <tinyNeoPixel_Static.h>
#include <avr/pgmspace.h> 

#define PIN PIN_PA1  // to NeoPixel LED
#define NLED 10 // number of NeoPixel LEDs

#define BRIGHTMAX  50
#define BRIGHTMAX2 (BRIGHTMAX * BRIGHTMAX)

#define DELAYVAL  20 // step time 20ms

uint8_t pixels[NLED * 3];
int16_t brightrgb;

tinyNeoPixel ledbar = tinyNeoPixel(NLED, PIN, NEO_GRB, pixels);


struct pix{
  unsigned char g; // green
  unsigned char r; // red
  unsigned char b; // blue
};

struct pix *planes;
uint8_t shift = 0;
uint8_t speed = 1;


struct proc{
  uint16_t t; // time
  int8_t   p; // position
  int8_t   d; // direction
  uint8_t  r; // roop
};

const struct proc inst[] PROGMEM = {
  {20, 0, 0, 1},
  {20, 0, 1, 1},
  {20, 1, 1, 1},
  {20, 2, 1, 1},
  {20, 3, 1, 1},
  {20, 4, 1, 1},
  {20, 5, 1, 1},
  {20, 6, 1, 1},
  {20, 7, 1, 1},
  {20, 8, 1, 1},
  {3000, 9, 1, 1},
  {45, 99, -1, 100},
  {20, 0, -1, 1},
  {20, 1, -1, 1},
  {20, 2, -1, 1},
  {20, 3, -1, 1},
  {20, 4, -1, 1},
  {20, 5, -1, 1},
  {20, 6, -1, 1},
  {20, 7, -1, 1},
  {20, 8, -1, 1},
  {300, 9, -1, 1},
  {10, -1, 0, 1}, // sleep
};

uint16_t bright[NLED] = {0};
int8_t gain[NLED] = {0};
int16_t starttime[NLED];

uint8_t shuffle[]= {6,3,5,8,1,0,4,9,2,7};


#define BOT 0
#define TOP 255
pix WheelB(uint8_t WheelPos){
  pix result;
  if(WheelPos < 85) {
    WheelPos *= 3;
    result.r = TOP - WheelPos;
    result.g = BOT + WheelPos;
    result.b = BOT;
  } else if(WheelPos < 170) {
    WheelPos = (WheelPos -85)*3;
    result.r = BOT;
    result.g = TOP - WheelPos;
    result.b = BOT + WheelPos;
  } else {
    WheelPos = (WheelPos -170)*3;
    result.r = BOT + WheelPos;
    result.g = BOT;
    result.b = TOP - WheelPos;
  }
  return (result);
}

void setplanes(uint16_t y, pix data, uint8_t br){
  planes[y].r = (uint16_t)data.r * br / 256;
  planes[y].g = (uint16_t)data.g * br / 256;
  planes[y].b = (uint16_t)data.b * br / 256;
}




void setup(){
  pinModeFast(PIN, OUTPUT);
  ledbar.clear();
  planes = (pix*)ledbar.getPixels();
  ledbar.show();
  
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  sleep_enable();
}


void loop() {
  static uint16_t tick = 0;
  static uint8_t ix = 0;
  static uint8_t fetch = 1;
  static uint8_t min = 0;
  static uint8_t port = 0;
  static uint8_t repeat = 0;
  struct proc p;

  if (fetch){
    fetch = 0;
    p.t = pgm_read_word_near(&inst[ix].t);
    p.p = pgm_read_byte_near(&inst[ix].p);
    p.d = pgm_read_byte_near(&inst[ix].d);
    p.r = pgm_read_byte_near(&inst[ix].r);
    tick = p.t;
    if (p.p == 99){
      port = 0;
      min = BRIGHTMAX / 2;
    } else if (p.p == -1){
      sleep_cpu();
    } else {
      port = p.p;
      min = 0;
    }
    gain[port] = p.d;
    repeat = p.r;
  }

  
  if (tick == 0){
    repeat--;
    if (repeat == 0){
      ix++;
      fetch = 1;
    } else{
      tick = p.t;
      port++;
      if (port >= NLED){
        port = 0;
      }
      gain[port] = p.d;
    }
  } else {
    tick--;
  }


  for (uint8_t i = 0; i < NLED; i++){
    bright[i] += gain[i];
    if (bright[i] >= BRIGHTMAX){
      gain[i] = 0;
    } else if (bright[i] == 0){
      gain[i] = 0;
    } else if (bright[i] <= min){
      gain[i] = 1;
    }
    uint8_t j = shuffle[i];
    setplanes(i, WheelB(j * 256 / NLED + shift), map(bright[j] * bright[j], 0, BRIGHTMAX2, 0, 255));
  }

  shift += speed;
  ledbar.show();

  delay(DELAYVAL);
}
